@@ -291,11 +291,11 @@ class Operation:
 
     @classmethod
     def build(cls: typing.Type[OperationType],
-              operands=[],
-              result_types=[],
-              attributes=[],
-              successors=[],
-              regions=[]) -> OperationType:
+              operands: List[Any] = None,
+              result_types: List[Any] = None,
+              attributes: Dict[str, Any] = None,
+              successors: List[Any] = None,
+              regions: List[Any] = None) -> OperationType:
         """Create a new operation using builders."""
         ...
 
@@ -378,6 +378,14 @@ class Operation:
             return self
         return self.parent.get_toplevel_object()
 
+    def is_ancestor(self, op: Union[Operation, Block, Region]) -> bool:
+        """Returns true if the operation is an ancestor of the operation, block, or region."""
+        if op is self:
+            return True
+        if op.parent is None:
+            return False
+        return self.is_ancestor(op.parent)
+
     def __eq__(self, other: Operation) -> bool:
         return self is other
 
@@ -432,22 +440,11 @@ class Block:
 
     def is_ancestor(self, op: Union[Operation, Block, Region]) -> bool:
         """Returns true if the block is an ancestor of the operation, block, or region."""
-        if isinstance(op, Block):
-            if op is self:
-                return True
-            if op.parent is None or op.parent.parent is None:
-                return False
-        if isinstance(op, Region):
-            if op.parent is None:
-                return False
-            op = op.parent
+        if op is self:
+            return True
         if op.parent is None:
             return False
-        if op.parent is self:
-            return True
-        if op.parent.parent is None or op.parent.parent.parent is None:
-            return False
-        return self.is_ancestor(op.parent.parent.parent)
+        return self.is_ancestor(op.parent)
 
     def insert_arg(self, typ: Attribute, index: int) -> BlockArgument:
         """
@@ -485,6 +482,10 @@ class Block:
             raise ValueError(
                 "Can't add to a block an operation already attached to a block."
             )
+        if operation.is_ancestor(self):
+            raise ValueError(
+                "Can't add an operation to a block contained in the operation."
+            )
         operation.parent = self
 
     def add_op(self, operation: Operation) -> None:
@@ -559,6 +560,10 @@ class Block:
 
     def verify(self) -> None:
         for operation in self.ops:
+            if operation.parent != self:
+                raise Exception(
+                    "Parent pointer of operation does not refer to containing region"
+                )
             operation.verify()
 
     def drop_all_references(self) -> None:
@@ -630,10 +635,92 @@ class Region:
                 return Region.from_operation_list(arg)
         raise TypeError(f"Can't build a region with argument {arg}")
 
+    @property
+    def ops(self) -> List[Operation]:
+        """
+        Get the operations of a single-block region.
+        Returns an exception if the region is not single-block.
+        """
+        if len(self.blocks) != 1:
+            raise ValueError(
+                "'ops' property of Region class is only available for single-block regions."
+            )
+        return self.blocks[0].ops
+
+    @property
+    def op(self) -> Operation:
+        """
+        Get the operation of a single-operation single-block region.
+        Returns an exception if the region is not single-operation single-block.
+        """
+        if len(self.blocks) != 1 or len(self.blocks[0].ops) != 1:
+            raise ValueError("'op' property of Region class is only available "
+                             "for single-operation single-block regions.")
+        return self.blocks[0].ops[0]
+
+    def _attach_block(self, block: Block) -> None:
+        """Attach a block to the region, and check that it has no parents."""
+        if block.parent is not None:
+            raise ValueError(
+                "Can't add to a region a block already attached to a region.")
+        if block.is_ancestor(self):
+            raise ValueError(
+                "Can't add a block to a region contained in the block.")
+        block.parent = self
+
     def add_block(self, block: Block) -> None:
         """Add a block to the region."""
+        self._attach_block(block)
         self.blocks.append(block)
-        block.parent = self
+
+    def insert_block(self, blocks: Union[Block, List[Block]],
+                     index: int) -> None:
+        """
+        Insert one or multiple blocks at a given index in the region.
+        The blocks should not be attached to another region.
+        """
+        if index < 0 or index > len(self.blocks):
+            raise ValueError(
+                f"Can't insert block in index {index} in a block with {len(self.blocks)} blocks."
+            )
+        if not isinstance(blocks, list):
+            blocks = [blocks]
+        for block in blocks:
+            self._attach_block(block)
+        self.blocks = self.blocks[:index] + blocks + self.blocks[index:]
+
+    def get_block_index(self, block: Block) -> int:
+        """Get the block position in a region."""
+        if block.parent is not self:
+            raise Exception("Block is not a child of the region.")
+        for idx, region_block in enumerate(self.blocks):
+            if region_block is block:
+                return idx
+        assert False, "Unexpected xdsl error"
+
+    def detach_block(self, block: Union[int, Block]) -> Block:
+        """
+        Detach a block from the region.
+        Returns the detached block.
+        """
+        if isinstance(block, Block):
+            block_idx = self.get_block_index(block)
+        else:
+            block_idx = block
+            op = self.blocks[block_idx]
+        if block.parent is not self:
+            raise Exception("Cannot detach block from a different region.")
+        block.parent = None
+        self.blocks = self.blocks[:block_idx] + self.blocks[block_idx + 1:]
+        return block
+
+    def erase_block(self, block: Union[int, Block], safe_erase=True) -> None:
+        """
+        Erase a block from the region.
+        If safe_erase is True, check that the block has no uses.
+        """
+        block = self.detach_block(block)
+        block.erase(safe_erase=safe_erase)
 
     def walk(self, fun: Callable[[Operation], None]) -> None:
         """Call a function on all operations contained in the region."""
@@ -643,6 +730,10 @@ class Region:
     def verify(self) -> None:
         for block in self.blocks:
             block.verify()
+            if block.parent != self:
+                raise Exception(
+                    "Parent pointer of block does not refer to containing region"
+                )
 
     def drop_all_references(self) -> None:
         """
@@ -664,9 +755,19 @@ class Region:
         """Move the blocks of this region to another region. Leave no blocks in this region."""
         region.blocks = self.blocks
         self.blocks = []
+        for block in region.blocks:
+            block.parent = region
 
     def get_toplevel_object(self) -> Union[Operation, Block, Region]:
         """Get the operation, block, or region ancestor that has no parents."""
         if self.parent is None:
             return self
         return self.parent.get_toplevel_object()
+
+    def is_ancestor(self, op: Union[Operation, Block, Region]) -> bool:
+        """Returns true if the region is an ancestor of the operation, block, or region."""
+        if op is self:
+            return True
+        if op.parent is None:
+            return False
+        return self.is_ancestor(op.parent)
